import django
if django.VERSION < (1, 6):
    # Expose for Django 1.5 and below (before DiscoverRunner)
    from .test_admin import *
    from .test_model_construction import *
    from .test_model_attributes import *
    from .test_model_inheritance import *
    from .test_forms import *
    from .test_urls import *
    from .test_query_count import *
